--	Copyright (C) 2004 by neuron
--	aagaande AT gmail DOT com
--	
--	This program is free software; you can redistribute it and--or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation; either version 2 of the License, or
--	(at your option) any later version.
--	
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--	
--	You should have received a copy of the GNU General Public License
--	along with this program; if not, write to the
--	Free Software Foundation, Inc.,
--	59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--	
--	Wording changed slightly on a few options, big thanks to next96 for cleaning up my localization and adding KoKR!
--	Marked a few things that might need rewording with "need verification"

local L = AceLibrary("AceLocale-2.2"):new("nQuestLog")

local options = {
	type='group',
	args = {
		resetPos = {
			type = 'execute',
			name = L["Reset position of minion"],
			desc = L["This is incase you lost it"],
			func = function()
				nQuestLogFrame:ClearAllPoints()
				nQuestLogFrame:SetPoint("CENTER",0,0)
				nQuestLog:SavePosition()
				nQuestLog:SetPosition()
				nQuestLog.db.profile["trackerLocked"] = false
			end
		},
		resetProfile = {
			type = 'execute',
			name = L["Reset Profile"],
			desc = L["This will reset the current profile"],
			order = 500,
			func = function()
				nQuestLog:ResetDB("profile")
				nQuestLogFrame:ClearAllPoints()
				nQuestLogFrame:SetPoint("CENTER",0,0)
				nQuestLog:SavePosition()
				nQuestLog:SetPosition()
				nQuestLog:RefreshTracker()
			end
		},
		toggleVisible = {
			type = 'toggle',
			name = L["Minion Visible"],
			desc = L["Note that minion will not show if you are not tracking any quests."],
			get = function()
				return nQuestLog.db.char["visible"]
			end,
			set = function(v)
				nQuestLog.db.char["visible"] = v
				nQuestLog:ToggleVisible(v)
			end
		},
		spacer = {
			type = 'header',
			name = ' ',
			order = 110,
		},
		settings = {
			type = 'group',
			order = 90,
			name = L["Settings"],
			desc = L["Settings"],
			args = {}
		}
	}
}

local function ToggleMenu(data)
	local func_Get,func_Set
	
	if (data.type == "toggle" or data.type == "range" or data.type == "text") then
		func_Get = function()
			if (data.reversed) then
				return not nQuestLog.db.profile[data.variable]
			end
			return nQuestLog.db.profile[data.variable]
		end
		func_Set = function(v)
			if (data.reversed) then
				v = not v
			end
			nQuestLog.db.profile[data.variable] = v
			nQuestLog:RefreshTracker()
			
			-- Custom stuff needed for only few things:
			if (data.variable == "width") then
				-- This refreshes width:
				nQuestLog:ToggleVisible(true)
			elseif (data.variable == "trackerGrowUp") then
				nQuestLog:SavePosition()
				nQuestLog:SetPosition()
			end
		end
	end
	
	return func_Get,func_Set
end

local AceProfile = {
	positionBottom = -1,
	positionLeft = -1,
	positionTop = -1,
}
local settingsTable = {
	-- This table is only used to generate AceProfile and the menu to change it
	{ name=L["Main settings"], type="header", variable="MainHeader" },
	{ name=L["Lock minion"], type="toggle", desc=L["Lock the minion to avoid dragging (drag by pulling top left)"], variable="trackerLocked", value=false },
	
	-- Layout stuff:
	{ name=L["Minion"], type="group" },
	{ name=L["Grow upwards"], type="toggle", desc=L["Grow minion up"], variable="trackerGrowUp", value=false },
	{ name=L["Width"], type="range", desc=L["Set width of frame"], variable="width", value=266, min=50,max=500 },
	{ name=L["Fontsize"], type="range", desc=L["Change font size for minion"], variable="fontSize", value=13, min=1,max=25,step=1 },
	{ name=L["Font outline"], type="toggle", desc=L["Add outline to minion font"], variable="fontOutline", value=false },
	
	--Zone Related:
	{ name=L["Zone"], type="group" },
	{ name=L["Show empty zones"], type="toggle", desc=L["Show hidden zones where you are not tracking any quests"], variable="showEmptyZones", value=true },
	{ name=L["Show Visible/Total quests for a zone"], type="toggle", desc=L["Shows <Zone> with X/X Visible"], variable="showZoneVisibleStatus", value=false },
	{ name=L["Show Zone level"], type="toggle", desc=L["Show zone level"], variable="showZoneLevel", value=true },
	{ name=L["Fontsize"], type="range", desc=L["Change font size for zone"], variable="fontSizeZone", value=14, min=1,max=25,step=1 },
	{ name=L["Padding"], type="range", desc=L["Change padding between zones"], variable="zonePadding", value=0, min=0,max=10,step=1 },
	
	--Quest Related:
	{ name=L["Quest"], type="group" },
	{ name=L["Show Quest Progress"], type="toggle", desc=L["Show quest progress (1/7 items total)"], variable="showQuestProgress", value=true },
	{ name=L["Padding"], type="range", desc=L["Change padding between quests"], variable="questPadding", value=0, min=0,max=10,step=1 },
	{ name=L["Auto show"], type="header", variable="MainHeader" },
	{ name=" - " .. L["New quests"], type="toggle", desc=L["Auto show new quests in minion"], variable="trackNewQuests", value=true },
	{ name=" - " .. L["When looting questitem"], type="toggle", desc=L["Auto show hidden quests when you loot a quest item for it"], variable="trackWhenLooting", value=true },
	
	--Quest objectives related:
	{ name=L["Quest objectives"], type="group" },
	{ name=L["Hide All Objectives"], type="toggle", desc=L["Hide all quest objectives by default"], variable="hideAllObjectives", value=true },
	{ name=L["Hide Completed Objectives"], type="toggle", desc=L["Hide completed objectives"], variable="hideCompletedObjectives", value=true },
	{ name=L["Show Description when no objectives"], type="toggle", desc=L["Show small quest description when no progress is available"], variable="showShortWhenNoObjectives", value=true },
}

-- Make options menu for all settings:
local function MakeAllSettings()
	local menuParent = options.args.settings.args
	for i,data in ipairs(settingsTable) do
		local func_Get,func_Set = ToggleMenu(data)
		
		-- First add menu
		local strName = data.name
		if (data.type ~= "header") then
			strName = "  " .. strName
		end
		
		if (data.type == "group") then
			menuParent = options.args.settings.args
			data.desc = strName .. " settings"
			data.variable = data.name
			data.args = {}
		end
		
		if (data.menuParent) then
			menuParent = data.menuParent
		end
		
		menuParent[data.name] = {
			type = data.type,
			name = strName,
			desc = data.desc, 
			order = i,
			get = func_Get,
			set = func_Set,
			min = data["min"],
			max = data["max"],
			step = data["step"],
			isPercent = data["isPercent"],
			validate = data["validate"],
			args = data.args,
			disabled = data.disabled,
		}
		
		if (data.type == "group") then
			menuParent = menuParent[data.variable].args
		end
		
		--Then add us to the RegisterDefaults db.
		AceProfile[data.variable] = data.value
	end
	
	settingsTable = nil
end

MakeAllSettings()

nQuestLog:RegisterDB("nQuestLogDB")
nQuestLog:RegisterDefaults('profile', AceProfile)

nQuestLog:RegisterDefaults('char', {
	visible = true,
	watchedQuests = {},
	watchedObjectives = {},
	hiddenZones = {},
})

nQuestLog.mainMenu = options

-- Chat menu
local nQuestLogConsole = AceLibrary("AceAddon-2.0"):new("AceHook-2.1", "AceConsole-2.0", "AceEvent-2.0")
local consoleMenu = {
	type = 'group',
	name = "nQuestLog",
	args = {
		resetPos = nQuestLog.mainMenu.args.resetPos,
		resetProfile = nQuestLog.mainMenu.args.resetProfile,
		show = nQuestLog.mainMenu.args.toggleVisible,
		lock = {
			type = 'toggle',
			name = L["Lock minion"],
			desc = L["Lock the minion to avoid dragging (drag by pulling top left)"],
			get = function()
				return nQuestLog.db.profile["trackerLocked"]
			end,
			set = function(v)
				nQuestLog.db.profile["trackerLocked"] = v
			end
		}
	}
}
nQuestLogConsole:RegisterChatCommand("/nquestlog", "/nql", consoleMenu)